@@ -153,7 +153,15 @@ data class CamtTransactionsCount(
      * Total number of transactions that were included in a report
      * or a statement.
      */
-    val downloadedTransactions: Int
+    val downloadedTransactions: Int,
+    /**
+     * Exceptions occurred while fetching transactions.  Fetching
+     * transactions can be done via multiple EBICS messages, therefore
+     * a failing one should not prevent other messages to be sent.
+     * This list collects all the exceptions that happened during the
+     * execution of a batch of messages.
+     */
+    var errors: List<Exception>? = null
 )
 
 /**
@@ -423,9 +431,10 @@ suspend fun fetchBankAccountTransactions(
     }
     /**
      * Collects transactions from the bank and stores the (camt)
-     * document into the database.
+     * document into the database.  This function tries to download
+     * both reports AND statements even if the first one fails.
      */
-    getConnectionPlugin(res.connectionType).fetchTransactions(
+    val errors: List<Exception>? = getConnectionPlugin(res.connectionType).fetchTransactions(
         fetchSpec,
         client,
         res.connectionName,
@@ -436,6 +445,7 @@ suspend fun fetchBankAccountTransactions(
     ingestFacadeTransactions(accountId, "taler-wire-gateway", ::talerFilter, ::maybeTalerRefunds)
     ingestFacadeTransactions(accountId, "anastasis", ::anastasisFilter, null)
 
+    ingestionResult.errors = errors
     return ingestionResult
 }
 
