@@ -5,7 +5,9 @@ import io.ktor.client.statement.*
 import io.ktor.http.*
 import io.ktor.server.testing.*
 import kotlinx.coroutines.runBlocking
+import org.jetbrains.exposed.sql.lowerCase
 import org.jetbrains.exposed.sql.transactions.transaction
+import org.junit.Ignore
 import org.junit.Test
 import tech.libeufin.sandbox.*
 import java.io.File
@@ -26,6 +28,24 @@ class SandboxCircuitApiTest {
         }
     }
 
+    @Test
+    fun accessAccountsTest() {
+        withTestDatabase {
+            prepSandboxDb()
+            testApplication {
+                application(sandboxApp)
+                val R = client.get("/demobanks/default/circuit-api/accounts/bar") {
+                    basicAuth("foo", "foo")
+                    expectSuccess = false
+                }
+                assert(R.status.value == HttpStatusCode.Forbidden.value)
+                client.get("/demobanks/default/circuit-api/accounts/bar") {
+                    basicAuth("admin", "foo")
+                    expectSuccess = true
+                }
+            }
+        }
+    }
     // Only tests that the calls get a 2xx status code.
     @Test
     fun listAccountsTest() {
@@ -96,8 +116,13 @@ class SandboxCircuitApiTest {
                         subject = "unused"
                         creationTime = 0L
                         tanChannel = SupportedTanChannels.FILE // change type to enum?
-                        account = "unused"
+                        account = "foo"
                         status = CashoutOperationStatus.PENDING
+                        cashoutAddress = "not used"
+                        buyAtRatio = "1"
+                        buyInFee = "1"
+                        sellAtRatio = "1"
+                        sellOutFee = "1"
                     }
                 }
                 R = client.get("/demobanks/default/circuit-api/cashouts") {
@@ -118,10 +143,57 @@ class SandboxCircuitApiTest {
                 val status = respJson.get("status").asText()
                 assert(status.uppercase() == "PENDING")
                 println(R.bodyAsText())
+                // Check that bar doesn't get foo's cash-out
+                R = client.get("/demobanks/default/circuit-api/cashouts?account=foo") {
+                    expectSuccess = false
+                    basicAuth("bar", "bar")
+                }
+                assert(R.status.value == HttpStatusCode.Forbidden.value)
+                // Check that foo can get its own
+                R = client.get("/demobanks/default/circuit-api/cashouts?account=foo") {
+                    expectSuccess = false
+                    basicAuth("foo", "foo")
+                }
+                assert(R.status.value == HttpStatusCode.OK.value)
             }
         }
     }
 
+    // Testing that only the admin can change an account legal name.
+    @Test
+    fun patchPerm() {
+        withTestDatabase {
+            prepSandboxDb()
+            testApplication {
+                application(sandboxApp)
+                val R =client.patch("/demobanks/default/circuit-api/accounts/foo") {
+                    contentType(ContentType.Application.Json)
+                    basicAuth("foo", "foo")
+                    expectSuccess = false
+                    setBody("""
+                        {
+                          "name": "new name",
+                          "contact_data": {},
+                          "cashout_address": "payto://iban/OUTSIDE"
+                        }
+                    """.trimIndent())
+                }
+                assert(R.status.value == HttpStatusCode.Forbidden.value)
+                client.patch("/demobanks/default/circuit-api/accounts/foo") {
+                    contentType(ContentType.Application.Json)
+                    basicAuth("admin", "foo")
+                    expectSuccess = true
+                    setBody("""
+                        {
+                          "name": "new name",
+                          "contact_data": {},
+                          "cashout_address": "payto://iban/OUTSIDE"
+                        }
+                    """.trimIndent())
+                }
+            }
+        }
+    }
     // Tests the creation and confirmation of a cash-out operation.
     @Test
     fun cashout() {
@@ -362,6 +434,23 @@ class SandboxCircuitApiTest {
         }
     }
 
+    // Tests the database RegEx filter on customer names.
+    @Ignore // Since no assert takes place.
+    @Test
+    fun customerFilter() {
+        withTestDatabase {
+            prepSandboxDb()
+            testApplication {
+                application(sandboxApp)
+                val R = client.get("/demobanks/default/circuit-api/accounts?filter=b") {
+                    basicAuth("admin", "foo")
+                    expectSuccess = true
+                }
+                println(R.bodyAsText())
+            }
+        }
+    }
+
     @Test
     fun tanCommandTest() {
         /**
