@@ -6,6 +6,8 @@ import io.ktor.http.*
 import io.ktor.server.request.*
 import io.ktor.server.response.*
 import io.ktor.server.routing.*
+import org.jetbrains.exposed.sql.deleteAll
+import org.jetbrains.exposed.sql.lowerCase
 import org.jetbrains.exposed.sql.transactions.transaction
 import tech.libeufin.sandbox.CashoutOperationsTable.uuid
 import tech.libeufin.util.*
@@ -36,7 +38,8 @@ data class CircuitCashoutRequest(
 data class ConfigResp(
     val name: String = "circuit",
     val version: String = SANDBOX_VERSION,
-    val ratios_and_fees: RatioAndFees
+    val ratios_and_fees: RatioAndFees,
+    val fiat_currency: String = "CHF" // FIXME: make configurable.
 )
 
 // After fixing #7527, the values held by this
@@ -66,7 +69,8 @@ data class CircuitContactData(
 
 data class CircuitAccountReconfiguration(
     val contact_data: CircuitContactData,
-    val cashout_address: String
+    val cashout_address: String,
+    val name: String? = null
 )
 
 data class AccountPasswordChange(
@@ -93,7 +97,9 @@ data class CashoutOperationInfo(
     val creation_time: Long, // milliseconds
     val confirmation_time: Long?, // milliseconds
     val tan_channel: SupportedTanChannels,
-    val account: String
+    val account: String,
+    val cashout_address: String,
+    val ratios_and_fees: RatioAndFees
 )
 
 data class CashoutConfirmation(val tan: String)
@@ -225,12 +231,6 @@ fun circuitApi(circuitRoute: Route) {
         // 404 if the operation is not found.
         if (op == null)
             throw notFound("Cash-out operation $operationUuid not found")
-        // 412 if the operation got already confirmed.
-        if (op.status == CashoutOperationStatus.CONFIRMED)
-            throw SandboxError(
-                HttpStatusCode.PreconditionFailed,
-                "Cash-out operation $operationUuid was already confirmed."
-            )
         /**
          * Check the TAN.  Give precedence to the TAN found
          * in the environment, for testing purposes.  If that's
@@ -251,7 +251,20 @@ fun circuitApi(circuitRoute: Route) {
          * NOTE: the funds availability got already checked when this operation
          * was created.  On top of that, the 'wireTransfer()' helper does also
          * check for funds availability.  */
+        val customer = maybeGetCustomer(user ?: throw SandboxError(
+            HttpStatusCode.ServiceUnavailable,
+            "This endpoint isn't served when the authentication is disabled."
+        ))
         transaction {
+            if (op.cashoutAddress != customer?.cashout_address) throw conflict(
+                "Inconsistent cash-out address: ${op.cashoutAddress} vs ${customer?.cashout_address}"
+            )
+            // 412 if the operation got already confirmed.
+            if (op.status == CashoutOperationStatus.CONFIRMED)
+                throw SandboxError(
+                    HttpStatusCode.PreconditionFailed,
+                    "Cash-out operation $operationUuid was already confirmed."
+                )
             wireTransfer(
                 debitAccount = op.account,
                 creditAccount = "admin",
@@ -284,19 +297,55 @@ fun circuitApi(circuitRoute: Route) {
             creation_time = maybeOperation.creationTime,
             confirmation_time = maybeOperation.confirmationTime,
             tan_channel = maybeOperation.tanChannel,
-            account = maybeOperation.account
+            account = maybeOperation.account,
+            cashout_address = maybeOperation.cashoutAddress,
+            ratios_and_fees = RatioAndFees(
+                buy_in_fee = maybeOperation.buyInFee.toFloat(),
+                buy_at_ratio = maybeOperation.buyAtRatio.toFloat(),
+                sell_out_fee = maybeOperation.sellOutFee.toFloat(),
+                sell_at_ratio = maybeOperation.sellAtRatio.toFloat()
+            )
         )
         call.respond(ret)
         return@get
     }
-    // Gets the list of all the cash-out operations.
+    // Gets the list of all the cash-out operations,
+    // or those belonging to the account given as a parameter.
     circuitRoute.get("/cashouts") {
-        call.request.basicAuth(onlyAdmin = true)
+        val user = call.request.basicAuth()
+        val whichAccount = call.request.queryParameters["account"]
+        /**
+         * Only admin's allowed to omit the target account (= get
+         * all the accounts) or to check other customers cash-out
+         * operations.
+         */
+        if (user != "admin" && whichAccount != user) throw forbidden(
+            "Ordinary users can only request their own account"
+        )
+        /**
+         * At this point, the client has the rights over the account(s)
+         * whose operations are to be returned.  Double-checking that
+         * Admin doesn't ask its own cash-outs, since that's not supported.
+         */
+        if (whichAccount == "admin") throw badRequest("Cash-out for admin is not supported")
+
+        // Preparing the response.
         val node = jacksonObjectMapper().createObjectNode()
         val maybeArray = node.putArray("cashouts")
-        transaction {
-            CashoutOperationEntity.all().forEach {
-                maybeArray.add(it.uuid.toString())
+
+        if (whichAccount == null) { // no target account, return all the cash-outs
+            transaction {
+                CashoutOperationEntity.all().forEach {
+                    maybeArray.add(it.uuid.toString())
+                }
+            }
+        } else { // do filter on the target account.
+            transaction {
+                CashoutOperationEntity.find {
+                    CashoutOperationsTable.account eq whichAccount
+                }.forEach {
+                    maybeArray.add(it.uuid.toString())
+                }
             }
         }
         if (maybeArray.size() == 0) {
@@ -333,7 +382,14 @@ fun circuitApi(circuitRoute: Route) {
                 "TAN channel '$tanChannel' not supported."
             )
         // check if the user contact data would allow the TAN channel.
-        val customer = getCustomer(username = user)
+        val customer: DemobankCustomerEntity? = maybeGetCustomer(username = user)
+        if (customer == null) throw internalServerError(
+            "Customer profile '$user' not found after authenticating it."
+        )
+        if (customer.cashout_address == null) throw SandboxError(
+            HttpStatusCode.PreconditionFailed,
+            "Cash-out address not found.  Did the user register via Circuit API?"
+        )
         if ((tanChannel == SupportedTanChannels.EMAIL.name) && (customer.email == null))
             throw conflict("E-mail address not found for '$user'.  Can't send the TAN")
         if ((tanChannel == SupportedTanChannels.SMS.name) && (customer.phone == null))
@@ -367,11 +423,18 @@ fun circuitApi(circuitRoute: Route) {
             CashoutOperationEntity.new {
                 this.amountDebit = req.amount_debit
                 this.amountCredit = req.amount_credit
+                this.buyAtRatio = ratiosAndFees.buy_at_ratio.toString()
+                this.buyInFee = ratiosAndFees.buy_in_fee.toString()
+                this.sellAtRatio = ratiosAndFees.sell_at_ratio.toString()
+                this.sellOutFee = ratiosAndFees.sell_out_fee.toString()
                 this.subject = cashoutSubject
                 this.creationTime = getUTCnow().toInstant().toEpochMilli()
                 this.tanChannel = SupportedTanChannels.valueOf(tanChannel)
                 this.account = user
                 this.tan = getRandomString(5)
+                this.cashoutAddress = customer.cashout_address ?: throw internalServerError(
+                    "Cash-out address for '$user' not found, after previous check succeeded"
+                )
             }
         }
         // Send the TAN.
@@ -444,7 +507,9 @@ fun circuitApi(circuitRoute: Route) {
         val username = call.request.basicAuth()
         val resourceName = call.getUriComponent("resourceName")
         throwIfInstitutionalName(resourceName)
-        allowOwnerOrAdmin(username, resourceName)
+        if (!allowOwnerOrAdmin(username, resourceName)) throw forbidden(
+            "User $username has no rights over $resourceName"
+        )
         val customer = getCustomer(resourceName)
         /**
          * CUSTOMER AND BANK ACCOUNT INVARIANT.
@@ -473,12 +538,24 @@ fun circuitApi(circuitRoute: Route) {
         ))
         return@get
     }
+
     // Get summary of all the accounts.
     circuitRoute.get("/accounts") {
         call.request.basicAuth(onlyAdmin = true)
+        val maybeFilter: String? = call.request.queryParameters["filter"]
+        /**
+         * Equip the given filter with left and right catch-all wildcards,
+         * otherwise use one catch-all wildcard.
+         */
+        val filter = if (maybeFilter != null) {
+            "%${maybeFilter}%"
+        } else "%"
         val customers = mutableListOf<Any>()
         transaction {
-            DemobankCustomerEntity.all().forEach {
+            DemobankCustomerEntity.find{
+                // like() is case insensitive.
+                DemobankCustomersTable.name.like(filter)
+            }.forEach {
                 if (it.cashout_address == null) {
                     logger.debug("Not listing account '${it.username}', as that" +
                             " misses the cash-out address " +
@@ -505,7 +582,9 @@ fun circuitApi(circuitRoute: Route) {
         val username = call.request.basicAuth()
         val customerUsername = call.getUriComponent("customerUsername")
         throwIfInstitutionalName(customerUsername)
-        allowOwnerOrAdmin(username, customerUsername)
+        if (!allowOwnerOrAdmin(username, customerUsername)) throw forbidden(
+            "User $username has no rights over $customerUsername"
+        )
         // Flow here means admin or username have the rights for this operation.
         val req = call.receive<AccountPasswordChange>()
         /**
@@ -527,9 +606,15 @@ fun circuitApi(circuitRoute: Route) {
             throw internalServerError("Authentication disabled, don't have a default for this request.")
         val resourceName = call.getUriComponent("resourceName")
         throwIfInstitutionalName(resourceName)
-        allowOwnerOrAdmin(username, resourceName)
+        if(!allowOwnerOrAdmin(username, resourceName)) throw forbidden(
+            "User $username has no rights over $resourceName"
+        )
         // account found and authentication succeeded
         val req = call.receive<CircuitAccountReconfiguration>()
+        // Only admin's allowed to change the legal name
+        if (req.name != null && username != "admin") throw forbidden(
+            "Only admin can change the user legal name"
+        )
         if ((req.contact_data.email != null) && (!checkEmailAddress(req.contact_data.email)))
             throw badRequest("Invalid e-mail address: ${req.contact_data.email}")
         if ((req.contact_data.phone != null) && (!checkPhoneNumber(req.contact_data.phone)))
